/**
 * This class represents a letter in a mail
 * merge system.  There can be any number of 
 * substitutions in the Letter data, of the
 * form $n where n is a decimal integer.
 * For example:
 * <pre>
 * Dear $1,
 * 
 *   We are repossessing your $2 because
 * you $3 
 *
 * $4 
 * Friendly Loan Company
 * </pre>
 * <p>
 * Normally, a class of this kind is initialised
 * from a File, but it can also be set up with
 * a String.
 * 
 * @author Neal Ziring, ziring@home.com
 */

import java.io.*;
import java.util.*;

public class Letter {
    /**
     * Substitution leader in a Letter.
     */
    public static final String LEADER = "$";

    private String contents;

    /**
     * Initialise a letter from a file.
     *
     * @throws IOException if an open or read error occurs
     */
    public Letter(File f) throws IOException {
	BufferedReader br;
	br = new BufferedReader(new FileReader(f));
	String line;
	StringBuffer buf = new StringBuffer();
	for(line = br.readLine(); line != null; line = br.readLine()) {
	    buf.append(line);
	    buf.append('\n');
	}
	br.close();
	contents = buf.toString();
    }

    /**
     * Initialise a letter from a String arra;
     *
     */
    public Letter(String s) {
	contents = s;
    }

    /**
     * Retrieve the raw text contents of this Letter
     */
    public String toString() { return contents; }


    /**
     * Perform substitutions on this letter, yielding
     * a String.  Note that the fields array is
     * expected to have an email address as field 0.
     * Field designators in the letter contents that
     * are null in the fields array, or outside the
     * field array bounds, will be left blank.
     */
    public String process(String [] fields) {
	StringBuffer buf = new StringBuffer(contents.length() * 2);
	
	int ip, cx, cv;
	String sv;
	char c1;

	for(ip = 0; (cv = contents.indexOf(LEADER, ip)) >= 0; ip = cv) 
	    {
		if (cv > ip)
		    buf.append(contents.substring(ip, cv));
		sv = "";
		for(cv += LEADER.length(); cv < contents.length(); cv++) {
		    c1 = contents.charAt(cv);
		    if (Character.isDigit(c1)) {
			sv += c1;
		    } else {
			break;
		    }
		}
		try {
		    int v = Integer.parseInt(sv);
		    if (v < fields.length && fields[v] != null) 
			buf.append(fields[v]);
		} catch (Exception e) { }
		// buf.append(c1);
	    }
	if (ip < contents.length()) {
	    buf.append(contents.substring(ip));
	}

	return buf.toString();
    }

    
    /* Uncomment this to unit test this class 
    public static void main(String [] args) {
	File lf = new File(args[0]);
	File df = new File(args[1]);
	Letter lt = null;
	MailMergeFile mmf = null;
	try {
	    mmf = new MailMergeFile(df);
	    lt = new Letter(lf);
	    int i,lim;
	    for(i = 0, lim = mmf.getCount(); i < lim; i++) {
		String [] fields = mmf.getFieldArray(i);
		String result = lt.process(fields);
		System.out.println("Result " + i + ":");
		System.out.println(result);
		System.out.println("-----");
	    }
	}
	catch (IOException e) {
	    System.err.println("Problem with test: " + e);
	    System.exit(0);
	}
    }
    */
}

