/**
 * Class to represent the data fields to be 
 * used in parameterising mail-merge letters.
 * An object of this class is normally to be
 * initialised from a data file.  The file 
 * has this format:
 * <p><pre>
 * address1|field1-1|field1-2|field1-3|....|field1-N
 * address2|field2-1|field2-2|field2-3|....|field2-N
 * </pre>
 * <p>
 * The file can also contain comments, of the form
 * <pre>
 * # comment 
 * </pre>
 * For more information, consult the README.TXT file
 * that should have accompanied this source code.
 *
 * @author Neal Ziring, ziring@home.com
 */

import java.io.*;
import java.util.*;

public class MailMergeFile {
    static final String DELIM = "|";
    private Vector contents;  /* vector of String vectors */
    
    /**
     * Create a MailMergeFile object from a specified
     * data file.  
     *
     * @throws IOException if any open or read errors occur
     */
    public MailMergeFile(File f) throws IOException
    {
	BufferedReader br;
	br = new BufferedReader(new FileReader(f));
	contents = new Vector();
	readFile(br);
	br.close();
    }

    protected void readFile(BufferedReader br) throws IOException {
	String line;
	StringTokenizer st;
	for(line = br.readLine(); line != null; line = br.readLine())
	    {
		if (line.length() < 3) continue;
		if (line.trim().startsWith("#")) continue;
		st = new StringTokenizer(line,DELIM);
		Vector v = new Vector();
		while(st.hasMoreTokens()) {
		    v.addElement(st.nextToken());
		}
		contents.addElement(v);
	    }
    }

    /**
     * Return the count of field lines we're holding.
     */
    public int getCount() { return contents.size(); }

    /**
     * Return a String array representing recipient n.
     * Return null if n < 0 or n > count of lines.
     */
    public String [] getFieldArray(int n) {
	if (n >= 0 && n < contents.size()) {
	    Vector v = ((Vector)(contents.elementAt(n)));
	    String [] ret = new String[v.size()];
	    for(int i = 0; i < ret.length; i++) {
		ret[i] = (String)(v.elementAt(i));
	    }
	    return ret;
	}
	else return null;
    }

    /**
     * Dump this MergeFieldFile object to the given
     * PrintStream, for debugging.
     */
    public void dump(PrintStream ps) {
	int i, j, lim;
	for(i = 0, lim = getCount(); i < lim; i++) {
	    ps.println("Line " + i + ":");
	    String [] fields = getFieldArray(i);
	    for(j = 0; j < fields.length; j++) {
		ps.println("\tField " + j + ": " + fields[j]);
	    }
	}
	ps.println("----");
	return;
    }

    /* Uncomment this to do unit testing on this class.
    public static void main(String [] args) {
	File f = new File(args[0]);
	MailMergeFile mmf = null;
	try {
	    mmf = new MailMergeFile(f);
	    System.out.println("Result was: ");
	    mmf.dump(System.out);
	}
	catch (IOException e) {
	    System.err.println("Error reading " + args[0] + ": " + e);
	    System.exit(0);
	}
    }
    */
}

