import java.net.*;
import java.io.*;
import java.util.*;

/**
 * MailSender is a simple class to send SMTP e-mail
 * based on the classes Letter and MailMergeFile.
 * This class is a Runnable, and the run() method
 * actually does the mailing.  It is also an
 * Observable, and sends progress reports to its
 * observers.
 * <p>
 * This class is not even close to a partial subset
 * implementation of SMTP.  For real production
 * applications, use of the JavaMail standard extension
 * would be a better choice.
 * 
 * @author Neal Ziring, ziring@home.com
 */

public class MailSender extends Observable implements Runnable
{

    private MailMergeFile data;
    private Letter        template;
    private String	  server;
    private String	  sender;
    private Thread	  myThread;
    private Socket	  sock;
    private int		  smtpPort;
    private boolean	  abort;

    /**
     * Create a MailSender object
     */
    public MailSender(String sv, int port, 
		      String from, 
		      Letter lt, MailMergeFile mmf)
    {
	sock = null;
	server = sv;
	smtpPort = port;
	sender = from;
	template = lt;
	data = mmf;
	abort = false;
	myThread = new Thread(this);
    }

    /**
     * Abort the run, if running.
     */
    public synchronized void abortRun() {
	abort = true;
    }

    /**
     * Start the run (can only be called once per object.)
     */
    public synchronized void start() {
	abort = false;
	myThread.start();
    }

    protected boolean doAbort() { return abort; }

    /**
     * Send all the letters to the mail server,
     * report activities to our observers.
     */
    public void run() {
	final String CRLF = "\r\n";
	BufferedReader br = null;
	PrintStream ps;
	String line;
	try {
	    sock = new Socket(server, smtpPort);
	    ps = new PrintStream(sock.getOutputStream());
	    br = new BufferedReader(new InputStreamReader(sock.getInputStream()));
	    line = br.readLine();
	}
	catch (Exception e) {
	    setChanged();
	    notifyObservers("Problem setting up connection: " + e);
	    return;
	}

	setChanged();
	notifyObservers("Connected to mail server " + server + ":" + smtpPort);
	if (doAbort()) {
	    try { ps.close(); br.close(); sock.close(); }
	    catch (Exception ex) { }
	    setChanged();
	    notifyObservers("Job aborted early, connection closed.");
	    return;
	}
	
	try {
	    for(int i = 0; i < data.getCount(); i++) {
		String [] fields = data.getFieldArray(i);

		ps.print("MAIL FROM:"); ps.print(sender); ps.print(CRLF);
		line = br.readLine();
		if (line.startsWith("25")) {
		    setChanged(); 
		    notifyObservers("Sending to: " + fields[0]);
		}
		else {
		    setChanged();
		    notifyObservers("Error from mail server (on MAIL): " + line);
		    break;
		}
		ps.print("RCPT TO:"); ps.print(fields[0]); ps.print(CRLF);
		line = br.readLine();
		if (!line.startsWith("25")) {
		    setChanged();
		    notifyObservers("Error from mail server (on RCPT): " + line);
		    break;
		}
		ps.print("DATA"); ps.print(CRLF);
		line = br.readLine();
		if (!line.startsWith("35")) {
		    setChanged();
		    notifyObservers("Error from mail server (on DATA): " + line);
		    break;
		}		    
		ps.print(template.process(fields));
		ps.print(CRLF);
		ps.print(".");
		ps.print(CRLF);
		setChanged();
		notifyObservers("Mail sent to " + fields[0] + " ok.");
		line = br.readLine();

		if (doAbort()) {
		    try { ps.close(); br.close(); sock.close(); }
		    catch (Exception ex) { }
		    setChanged();
		    notifyObservers("Job aborted early, connection closed.");
		    return;
		}
	    }
	}
	catch (Exception e2) {
	    setChanged();
	    notifyObservers("Problem reading information from server: " + e2);
	}

	try {
	    br.close();
	    ps.close();
	    sock.close();
	}
	catch (Exception e3) {
	    setChanged();
	    notifyObservers("Problem closing down connection: " + e3);
	}
	return;
    }

}

