import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;



/**
 * GUI for mail merge tool - this GUI has
 * basically three parts: a menu bar, a
 * control area, and a log area. 
 * This GUI also implements Observer, so
 * that it can easily keep track of 
 * it stuff going on in Observable
 * objects.
 * 
 * @author Neal Ziring, ziring@home.com
 */

public class MergeGui implements Observer
{
    public static int SMTP_PORT = 25;

    /* primary GUI elements */
    Frame	mainFrame;
    MenuBar	bar;
    Menu	fileMenu;
    Panel	topPanel;
    TextArea	logArea;
    TextField	serverField;
    TextField	senderField;
    TextField	letterFileField;
    TextField	dataFileField;
    FileDialog	letterFiler;
    FileDialog  dataFiler;
    
    /**
     * Create the interface, all text
     * fields start blank.
     */
    public MergeGui() {
	mainFrame = new Frame("Mail Merge GUI 0.2");
	bar = new MenuBar();
	fileMenu = new Menu("File");
	
	MenuItem letterBrowseItem = new MenuItem("Browse for Letter..");
	fileMenu.add(letterBrowseItem);
	MenuItem dataBrowseItem = new MenuItem("Browse for Data..");
	fileMenu.add(dataBrowseItem);
	fileMenu.addSeparator();
	MenuItem processItem = new MenuItem("Do the mailing!");
	fileMenu.add(processItem);
	fileMenu.addSeparator();
	MenuItem quitItem = new MenuItem("Exit");
	fileMenu.add(quitItem);

	bar.add(fileMenu);
	mainFrame.setMenuBar(bar);

	topPanel = new Panel();
	topPanel.setLayout(new GridLayout(4,2,5,3));
	
	serverField = addPanelField(topPanel, "SMTP mail server host:");
	senderField = addPanelField(topPanel, "Sender e-mail address:");
	letterFileField = addPanelField(topPanel, "Form letter template file:");
	dataFileField = addPanelField(topPanel, "Recipient field data file:");
	
	mainFrame.add("North", topPanel);
	
	logArea = new TextArea("Please set all the fields before\ndoing the mailing.\n\n",20,42);
	logArea.setEditable(false);
	mainFrame.add("Center", logArea);

	letterFiler = new FileDialog(mainFrame, "Select Letter Template",
				     FileDialog.LOAD);
	dataFiler = new FileDialog(mainFrame, "Select Data Table",
				   FileDialog.LOAD);

	/* set up the event handlers */
	mainFrame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    });

	letterBrowseItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    letterFiler.show();
		    if (letterFiler.getFile() != null) {
			File p = new File(letterFiler.getDirectory(),
					  letterFiler.getFile());
			letterFileField.setText(p.getPath());
			update(null, "Set letter file to " + p.getPath());
		    }
		}
	    });
	dataBrowseItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    dataFiler.show();
		    if (dataFiler.getFile() != null) {
			File p = new File(dataFiler.getDirectory(),
					  dataFiler.getFile());
			dataFileField.setText(p.getPath());
			update(null, "Set letter file to " + p.getPath());
		    }
		}
	    });

	processItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    processMail();
		}
	    });	

	quitItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    System.exit(0);
		}
	    });

	mainFrame.pack();
    }
    
    protected TextField addPanelField(Panel parent, String lbl) {
	Label l = new Label(lbl, Label.RIGHT);
	TextField ret = new TextField(26);
	parent.add(l);
	parent.add(ret);
	return ret;
    }

    
    /**
     * Perform the mail processing, after first checking
     * that all the fields are filled in.
     */
    public void processMail() {
	String lfile = letterFileField.getText();
	String dfile = dataFileField.getText();
	String server= serverField.getText().trim();
	String sender= senderField.getText().trim();
	MailMergeFile mmf = null;
	Letter lt = null;

	if (lfile.length() == 0) {
	    update(null, "Need to fill in letter file field.");
	    return;
	}
	if (dfile.length() == 0) {
	    update(null, "Need to fill in data file field.");
	    return;
	}
	if (server.length() == 0) {
	    update(null, "Need to fill in the server name.");
	    return;
	}
	if (sender.length() == 0) {
	    update(null, "Need to fill in the sender e-mail address.");
	    return;
	}

	try {
	    mmf = new MailMergeFile(new File(dfile));
	    lt = new Letter(new File(lfile));
	    
	    MailSender ms = new MailSender(server, SMTP_PORT, sender, 
					   lt, mmf);
	    ms.addObserver(this);
	    Thread tr = new Thread(ms);
	    tr.start();
	    update(null, "Sorry, full sending not yet implemented.");
	}
	catch (IOException e) {
	    update(null, "Could not process files.");
	    update(null, e);
	}
	return;
    }

    /**
     * Append an arg string to the log text area.
     */
    public void update(Observable o, Object arg) {
	logArea.append(arg.toString());
	logArea.append("\n");
    }

    public static void main(String [] args) {
	MergeGui gui;

	gui = new MergeGui();
	gui.mainFrame.show();
    }
}

    
